import math
import numpy
import gzip
from glob import iglob

from prosci.util.protein import Protein, ResidueList, Pdb
from prosci.util.pdb import residueLetter, residueCode


def dist(a, b):
    return math.sqrt((a[0]-b[0])**2 + (a[1]-b[1])**2 + (a[2]-b[2])**2)

def angle(a, b):
  return math.acos(numpy.dot(a, b)/(numpy.linalg.norm(a)*numpy.linalg.norm(b)))

def grow_cb_dictionary(chain, cb_dictionary):
  for res in chain:
    if res.CB and res.CA and res.C and res.N:
      n_direction = res.N.xyz - res.CA.xyz
      n_direction /= numpy.linalg.norm(n_direction)
      c_direction = res.C.xyz - res.CA.xyz
      c_direction /= numpy.linalg.norm(c_direction)
      norm_direction = numpy.cross(n_direction, c_direction)
      norm_direction /= numpy.linalg.norm(norm_direction)
      norm2_direction = numpy.cross(n_direction, norm_direction)
      norm2_direction /= numpy.linalg.norm(norm_direction)
      
      cb_vector = res.CB.xyz-res.CA.xyz
      cb_dist = numpy.linalg.norm(cb_vector)
      cb_direction = cb_vector / cb_dist
      
      n_factor = numpy.dot(cb_direction, n_direction)
      norm_factor = numpy.dot(cb_direction, norm_direction)
      norm2_factor = numpy.dot(cb_direction, norm2_direction)
      
      print "%s %.3f %.3f %.3f %.3f" % (res.res, n_factor, norm_factor, norm2_factor, cb_dist)
      
      if not res.res in cb_dictionary:
        cb_dictionary[res.res] = [0.0, 0.0, 0.0, 0.0, 0]
      values = cb_dictionary[res.res]
      
      values[0] += n_factor
      values[1] += norm_factor
      values[2] += norm2_factor
      values[3] += cb_dist
      values[4] += 1


def test_rebuilt_CBs(chain, cb_dictionary):
  for res in chain:
    if res.CB and res.CA and res.C and res.N:
      # regenerate CB position and check it against the real position
      n_direction = res.N.xyz - res.CA.xyz
      n_direction /= numpy.linalg.norm(n_direction)
      c_direction = res.C.xyz - res.CA.xyz
      c_direction /= numpy.linalg.norm(c_direction)
      norm_direction = numpy.cross(n_direction, c_direction)
      norm_direction /= numpy.linalg.norm(norm_direction)
      norm2_direction = numpy.cross(n_direction, norm_direction)
      norm2_direction /= numpy.linalg.norm(norm_direction)
      
      try:
        n_factor, norm_factor, norm2_factor, cb_dist = cb_dictionary[res.res]
      except ValueError:
        n_factor, norm_factor, norm2_factor, cb_dist, count = cb_dictionary[res.res]
        n_factor /= count
        norm_factor /= count
        norm2_factor /= count
        cb_dist /= count
      
      cb_vector = (n_direction * n_factor) + (norm_direction * norm_factor) + (norm2_direction * norm2_factor)
      cb_vector /= numpy.linalg.norm(cb_vector)
      cb_vector *= cb_dist
      
      cb_xyz = res.CA.xyz + cb_vector
      print res.chain, res.ires, res.inscode, res.res, dist(cb_xyz, res.CB.xyz)


cb_dictionary = {}

with open("cullpdb_pc20_res1.6_R0.25_d120618_chains2029") as f:
  f.next()
  for line in f:
    fields = line.split()
    pdb = fields[0][:4].lower()
    chain = fields[0][4:5]
    fname = "/data/pegasus/DATABASES/pdb/%s/pdb%s.ent.gz" % (pdb[1:3], pdb)
    print fname
    try:
      prot = Protein(Pdb(gzip.open(fname)))
      prot_chain = prot.get_chain(chain)
    except:
      continue
    grow_cb_dictionary(prot_chain, cb_dictionary)


for rescode in cb_dictionary:
  values = cb_dictionary[rescode]
  for i,v in enumerate(values[:-1]):
    values[i] = v / values[-1]
  print rescode, values
  values.pop()

f = open("cb_dictionary.txt", "w")
f.write(str(cb_dictionary))
f.write("\n")
f.close()
