#!/usr/bin/env python

import sys
import os.path

from prosci.common import splitpath
from prosci.imembrane.builder import Classifier


def get_chainbreaks(dbdir, entryid):
  #stream  = os.popen("cat %s | sort -u -f -s -k4,4 | egrep -v '^#|^$' | sort -n | cut -f1,4" % (filename))
  
  cl = Classifier(os.path.dirname(dbdir), entryid)

  indeces = cl.get_chain_boundaries()
  
  data = []
  for i in indeces:
    cgnum = cl[i][Classifier.CGNUM]
    chain = cl[i][Classifier.CHAIN]
    data.append((cgnum, chain))
  return data


def parse_file(pdb_file, pdb_code):
        chainbreaks = get_chainbreaks(os.path.dirname(pdb_file), pdb_code)
        #print chainbreaks
        infile = open(pdb_file)
        outfiles = []
        chainids = []
        atm = []
        fields = None
        for line in infile:
            if not line.startswith('ATOM'):
                continue
            
            resnum = int(line[22:26].strip())
            
            if chainbreaks and chainbreaks[0][0] <= resnum:
                chainids.append(chainbreaks[0][1])
                #print chainids
                chainbreaks.pop(0)
                if atm:
                    outfiles.append(atm)
                    atm = []
            
            #newline = line[:21] + chainbreaks[0][1] + line[22:]
            atm.append(line)

        if atm:
          outfiles.append(atm)

        return outfiles, chainids


def splitchains(pdb_file, doprint=False):
    path, basename, ext = splitpath(pdb_file)

    pdb_code = basename.split('-')[0]

    chaindata, chainids = parse_file(pdb_file, pdb_code)
    
    if len(chaindata) != len(chainids):
      sys.stderr.write("Count of chain data and ids not the same.\n")
      sys.stderr.write("%s\n" % (chainids))
      sys.stderr.write("%s\n" % (len(chaindata)))
      sys.stderr.write("%s\n" % (len(chainids)))
      sys.exit(1)
    
    
    # Make sure duplicate chains are given different chain IDs.
    # The format is "X.N" for any chain X, where N is an incrementing number for
    # each occurrence of that chain
    #
    chaincounts = dict()
    for i in xrange(len(chainids)):
      c = chainids[i]
      if chaincounts.has_key(c):
        count = chaincounts[c] + 1
      else:
        count = 0
      chaincounts[c] = count

      chainids[i] = c + str(count)
    
    
    for i in xrange(len(chaindata)):
      data = chaindata[i]
      c = chainids[i]
      newfilename = "%s.%s.%s.%s.atm" % (pdb_code[:4], pdb_code[4:].replace(".", ""), c[:1], c[1:])
      f = open(newfilename, 'w')
      for line in data:
        f.write(line)
      f.close()
      if os.path.isfile(newfilename):
        if doprint:
          print newfilename
      else:
        sys.stderr.write("ERROR creating file: %s\n" % (newfilename))
