#!/usr/bin/perl -T


use strict;
use CGI qw/:standard/;
use Cwd  qw(abs_path);
use File::Basename;

my $EXECDIR = abs_path(dirname($0)); $EXECDIR =~ /^(.*)$/; $EXECDIR = $1;
require("$EXECDIR/cfg.pl");
my %CONFIG = %{get_config()};

my $inputnum = 1; # this is just for display purposes, numbering the input boxes

my $currentyear = get_current_year();

print 
    header,
    start_html(
        -title=>$CONFIG{'APPNAME'},
        -author=>'Sebastian Kelm',
        -meta=>{
            'keywords'=>'MEDELLER, membrane protein, homology modelling, iMembrane, membrane insertion, protein structure, protein sequence, annotation, alignment, superposition, membrane layer, membrane contact, structural bioinformatics, protein informatics, oxford university, department of statistics, OPIG',
            'copyright'=>"copyright 2008-$currentyear Sebastian Kelm, Jiye Shi, Charlotte M. Deane",
            'description'=>'Homology-Based Membrane Insertion and Coordinate Generation for Membrane Proteins'
            },
        -style=>{'src'=>$CONFIG{'STYLESHEET'}, 'code'=>'
            body
            {
              padding:20px;
              margin:0px;
            }'},
        -script=>{'code'=>
"
function block(element, value)
{
  v = value ? 'block' : 'none';
  if (document.layers)
  {
    document.layers[element].display = v;
  }
  else if (document.all)
  {
    document.all[element].style.display = v;
  }
  else if (document.getElementById)
  {
    document.getElementById(element).style.display = v;
  }
}

function inline(element, value)
{ 
  v = value ? 'inline' : 'none';
  if (document.layers)
  {
    document.layers[element].display = v;
  }
  else if (document.all)
  {
    document.all[element].style.display = v;
  }
  else if (document.getElementById)
  {
    document.getElementById(element).style.display = v;
  }
}
   "},
    -onload=>''
    ), '
    
    <div align="center"><table><tr><td> <!-- begin CENTER -->
    
    '.$CONFIG{'HTML_MINIMAL_HEADER'}.'
    <div class="switchapp"><a href="'.$CONFIG{'SERVERURL'}.'">[Switch Application]</a></div>
    
    <div class="mainmargin"> <!-- begin MAINMARGIN -->
    
    <p>&nbsp;</p>
    
    <p class="usagepolicy">For Academic Use Only<br/>&nbsp;</p>
    
    ',start_form('POST', $CONFIG{'SUBMITURL'});
    
    unless ($CONFIG{'APPNAME'} eq 'MEDELLER')
    {
      print '

    <div class="inputgroup">
      <div class="inputnumber">
        '.($inputnum++).'
      </div>
      <h3>'.html_pretty('Input Type:').'</h3>
      <div class="sidemargin">
        
        <div id="inputtype">
          <p>Is your input a protein structure or a sequence?</p>
          <div class="sidemargin">
            <label><input type="radio" id="input_type_structure"  name="input_type" value="structure" checked="checked" onclick="
              block(\'structureinput\', true); block(\'sequenceinput\', false); inline(\'imem_search_text_struc\', true); inline(\'imem_search_text_seq\', false);" />structure</label>
            <label><input type="radio" id="input_type_sequence"  name="input_type" value="sequence" onclick="
              block(\'structureinput\', false); block(\'sequenceinput\', true); inline(\'imem_search_text_struc\', false); inline(\'imem_search_text_seq\', true);" />sequence</label>
          </div>
        </div>
      
      </div>
    </div>
      ';
    }
    
    print '
    <div class="inputgroup" id="sequenceinput" '.(($CONFIG{'APPNAME'} eq 'MEDELLER') ? '' : 'style="display:none;"').'>
      <div class="inputnumber">
        '.(($CONFIG{'APPNAME'} eq 'MEDELLER') ? $inputnum++ : $inputnum).'
      </div>
      <h3>'.html_pretty(($CONFIG{'APPNAME'} eq 'MEDELLER') ? 'Input Target-Template Alignment:' : 'Input Sequence:').'</h3>
      <div class="sidemargin">
        
        <p>How would you like to provide your sequence '.(($CONFIG{'APPNAME'} eq 'MEDELLER') ? 'alignment' : '').' (PIR or <a target="_blank" href="http://en.wikipedia.org/wiki/Fasta_format">FASTA</a>)?</p>
        <div class="sidemargin">
          <label><input type="radio" id="seq_method_file"  name="seq_method" value="file" checked="checked" onclick="
            block(\'seq_file\', true); block(\'seq_text\', false);" />Upload a file</label>
          <label><input type="radio" id="seq_method_text"  name="seq_method" value="text" onclick="
            block(\'seq_file\', false); block(\'seq_text\', true);" />Copy&amp;paste the file contents</label>
        </div>
        <div id="seq_file">
          <p>Choose a file to upload from your hard drive:</p>
          <div class="sidemargin">
            ',filefield('seq_file','',50,80),'
          </div>
        </div>
        <div id="seq_text" style="display:none;">
          <p>Paste your sequence data into the following text field:</p>
          <div class="sidemargin">
          ',textarea('seq_text','',10,70),'
          </div>
        </div>';
    if ($CONFIG{'APPNAME'} eq 'MEDELLER')
    {
      print '
        <div class="sidemargin">
          <div class="note">
            Please label the target and template sequences as "&gt;target" and<br>"&gt;template", respectively (without the quotes).
          </div>
          <div class="note">
            <a href="example_input/target_template_alignment.fasta">Example input file</a>, a sequence alignment of PDB entries 2JAFA (target) and 2VT4A (template)
          </div>
        </div>';
    }
    else
    {
      print '
        <div class="sidemargin">
          <div class="note">
            <a href="example_input/2JAFA.fasta">Example input file</a>, the sequence of PDB entry 2JAFA
          </div>
        </div>';
    }
    print '
      </div>
      <span class="clear"></span>
    </div>
    
    
    <div class="inputgroup" id="structureinput">
      <div class="inputnumber">
        '.($inputnum++).'
      </div>
      <h3>'.html_pretty(($CONFIG{'APPNAME'} eq 'MEDELLER') ? 'Input Template Structure:' : 'Input Structure:').'</h3>
      <div class="sidemargin">
        
        How would you like to provide your protein structure (<a target="_blank" href="http://en.wikipedia.org/wiki/Protein_Data_Bank_(file_format)">PDB</a>, one chain only)?<br/>
        <div class="sidemargin">
          <label><input type="radio" id="struc_method_pdbid" name="struc_method" value="pdbid" checked="checked" onclick="
            block(\'struc_pdbid\', true); block(\'struc_file\', false); block(\'struc_text\', false);" />Enter a PDB code</label>
          <label><input type="radio" id="struc_method_file"  name="struc_method" value="file" onclick="
            block(\'struc_pdbid\', false); block(\'struc_file\', true); block(\'struc_text\', false);" />Upload a file</label>
          <label><input type="radio" id="struc_method_text"  name="struc_method" value="text" onclick="
            block(\'struc_pdbid\', false); block(\'struc_file\', false); block(\'struc_text\', true);" />Copy&amp;paste the file contents</label>
        </div>
        <div id="struc_pdbid">
          <p>Enter a valid PDB code:</p>
          <div class="sidemargin">
            ',textfield('struc_pdbid','',5,5),'
            <div class="note">
             Enter 5 characters, the standard PDB code followed by the chain identifier, e.g. &quot;',(($CONFIG{'APPNAME'} eq 'MEDELLER') ? '2VT4A' : '2JAFA'),'&quot;<br />
             Note that chain identifiers are case sensitive.
            </div>
          </div>
        </div>
        <div id="struc_file" style="display:none;">
          <p>Choose a file to upload from your hard drive:</p>
          <div class="sidemargin">
            ',filefield('struc_file','',50,80),'
            <div class="note">
              ',(($CONFIG{'APPNAME'} eq 'MEDELLER') ? '<a href="example_input/template_2VT4A.pdb">Example input file</a>, the structure of PDB entry 2VT4A)' : '<a href="example_input/2JAFA.pdb">Example input file</a>, the structure of PDB entry 2JAFA'),'
            </div>
          </div>
        </div>
        <div id="struc_text" style="display:none;">
          <p>Paste your structure data into the following text field:</p>
          <div class="sidemargin">
          ',textarea('struc_text','',10,70),'
            <div class="note">
              ',(($CONFIG{'APPNAME'} eq 'MEDELLER') ? '<a href="example_input/template_2VT4A.pdb">Example input file</a>, the structure of PDB entry 2VT4A)' : '<a href="example_input/2JAFA.pdb">Example input file</a>, the structure of PDB entry 2JAFA'),'
            </div>
          </div>
        </div>
      </div>
      <span class="clear"></span>
    </div>
    
    
    <div class="inputgroup">
      <div class="inputnumber">
        '.($inputnum++).'
      </div>
      <h3>'.html_pretty('Options:').'</h3>
      <div class="sidemargin">
        ';
        
if ($CONFIG{'APPNAME'} eq 'MEDELLER')
{
  print '
        <div id="opt_med_showmore" style="display:none;"><a href="javascript:void();" onclick="block(\'opt_med_more\', true); block(\'opt_med_showmore\', false);">Show MEDELLER options ...</a></div>
        <div id="opt_med_more">
          <p><a href="javascript:void();" onclick="block(\'opt_med_showmore\', true); block(\'opt_med_more\', false);">Hide MEDELLER options ...</a></p>
          
          <p>
            Model type:<br />
            <label><input type="radio" id="med_model_core"     name="med_model" value="core"                       />Core only</label>
            <label><input type="radio" id="med_model_loops"    name="med_model" value="loops"                      />Core &amp; Loops</label>
            <label><input type="radio" id="med_model_complete" name="med_model" value="complete" checked="checked" />Complete</label>
          </p>
          <div>
            <label><input type="checkbox" id="med_bigcore" name="med_bigcore" value="yes" />Start modelling from a bigger &quot;minimal core&quot;.</label>
            <div class="sidemargin">
              <div class="note">
                The standard &quot;minimal core&quot; includes only residues in the core (T) layer of the membrane.<br />
                If this option is enabled, residues in the outer (H) layers are also included. This is useful<br />
                if your protein only partially penetrates the membrane.
              </div>
            </div>
          </div>
          &nbsp;
        </div>
  ';
}

print '
        <div id="opt_imem_showmore"><a href="javascript:void();" onclick="block(\'opt_imem_more\', true); block(\'opt_imem_showmore\', false);">Show iMembrane options ...</a></div>
        <div id="opt_imem_more" style="display:none;">
          <p><a href="javascript:void();" onclick="block(\'opt_imem_showmore\', true); block(\'opt_imem_more\', false);">Hide iMembrane options ...</a></p>
          
          <p>
            Search type:<br />
            <label><input type="radio" id="imem_search_auto"     name="imem_search" value="auto"     checked="checked" onclick="block(\'opt_fast\', true);"  />auto</label>
            <label><input type="radio" id="imem_search_fast"     name="imem_search" value="fast"                       onclick="block(\'opt_fast\', true);"  />fast</label>
            <label><input type="radio" id="imem_search_thorough" name="imem_search" value="thorough"                   onclick="block(\'opt_fast\', false);" />thorough</label>
          </p>
          <div class="note">
            <ul>
              <li>"Auto" will run a fast search. If no hits are found, a thorough search is run.</li>
              <li>"Fast" will perform a sequence search with BLAST. This may not yield any hits.</li>
              <li>"Thorough" will search the database by <span id="imem_search_text_struc">structure superposition</span><span id="imem_search_text_seq" style="display:none;">alignment with MUSCLE</span>. This is relatively slow.</li>
            </ul>
            &nbsp;
          </div>
          
          <div id="opt_fast">
            <div class="prompt">PSI-BLAST search parameters:</div>
            <ul>
              <li>E-value cut-off (accepting hits):',textfield('imem_cutoff_e','1.0',10,10),'</li>
              <li>E-value cut-off (addition to profile): ',textfield('imem_cutoff_e2','0.002',10,10),'</li>
              <li>Iterations:',textfield('imem_blastiterations','3',2,2),'</li>
            </ul>
          </div>
          <div id="opt_seq">
            <div class="prompt">Sequence-based parameters:</div>
            <ul>
              <li>Percentage identity cut-off: ',textfield('imem_cutoff_pid','0.00',5,5),'</li>
              <li>Percentage annotated cut-off: ',textfield('imem_cutoff_pannot','0.5',5,5),'</li>
            </ul>
          </div>
          <div id="opt_struc">
            <div class="prompt">Structure-based parameters:</div>
            <ul>
              <li>TM-score cut-off: ',textfield('imem_cutoff_tm','0.5',5,5),'</li>
              <li>RMSD cut-off: ',textfield('imem_cutoff_rmsd','10.0',5,5),'</li>
            </ul>
          </div>
          
          <p class="note">
            Note: All searches are restricted to the top 10 hits.
          </p>
        </div>
        
      </div>
      <span class="clear"></span>
    </div>
    
    <input type="hidden" name="app" value="'.$CONFIG{'APPNAME'}.'" />
    
    <div class="inputgroup">
      <div class="inputnumber">
        '.($inputnum++).'
      </div>
      <h3>'.html_pretty('Submit:').'</h3>
      <div class="sidemargin">
        <p>Please press the submit button <em>once</em> only - uploading takes time. Thank you.</p>
        ',submit,'
      </div>
      <span class="clear"></span>
    </div>
    
    ',end_form,qq(
    
    <p class="center">
      Already have a result ID? You can <a href=").internallink('archive').qq(">review your previous results</a>.
    </p>
    
    <p class="center">
      Struggling? Have a look at the <a href=").internallink('help').qq(">Help page</a>.
    </p>
    
    <p class="center">
      <a href=").internallink('queue').qq(&amp;result_id=example_medeller">View an example result</a>.
    </p>
    
    <p>&nbsp;</p>
    
    </div> <!-- end MAINMARGIN -->
    
    <div class="footer">
    <a target="_blank" href="http://www.stats.ox.ac.uk/proteins/">Protein Informatics Group</a> &nbsp; | &nbsp; &copy; <a target="_blank" href="http://www.stats.ox.ac.uk/~kelm/">Sebastian Kelm</a>, Jiye Shi, <a target="_blank" href="http://www.stats.ox.ac.uk/~deane/">Charlotte M. Deane</a>, 2008-$currentyear
    </div>
    
    </td></tr></table></div> <!-- end CENTER -->

    ),
    "\n";
#~ 
#~ print <<END;
  #~ <!-- Page statistics by Google Analytics -->
  #~ <script type="text/javascript">
  #~ var gaJsHost = (("https:" == document.location.protocol) ? "https://ssl." : "http://www.");
  #~ document.write(unescape("%3Cscript src='" + gaJsHost + "google-analytics.com/ga.js' type='text/javascript'%3E%3C/script%3E"));
  #~ </script>
  #~ <script type="text/javascript">
  #~ var pageTracker = _gat._getTracker("UA-6136402-1");
  #~ pageTracker._trackPageview();
  #~ </script>
#~ END
print end_html;

#~ <div class="note">You can also provide a single target sequence, instead of an alignment, but this is not<br />
            #~ recommended. In that case, the target and template will be aligned using pure sequence alignment.</div>
