#!/usr/bin/perl -T
#
# This CGI programme takes protein PDB codes/files or FASTA protein sequences as input.
#
# It gives back an annotated alignment between the input and CGDB membrane protein sequences.
# This can be in sequence or structure format.
#
# NOTE: This script runs in Perl's "Taint" mode ('-T' flag in first line).
#


use strict;
use Cwd  qw(abs_path);
use File::Basename;
use CGI;
use File::Glob ':glob';
use Text::Wrap;

my $EXECDIR = abs_path(dirname($0)); $EXECDIR =~ /^(.*)$/; $EXECDIR = $1;
require("$EXECDIR/cfg.pl");
my %CONFIG = %{get_config()};

my $q = new CGI;  # CGI object
my $SELFURL = $q->url();



###################################
#### SETTINGS AND PREPARATIONS ####
###################################


my $OUTPUTDIR;


my $result_id  = $q->param('result_id');
my ($job_status, $status_time, $status_message) = job_getstatus($result_id);
my $t = time()-$status_time;
$status_message = wrap('', '', $status_message);

unless(defined($result_id) and ($result_id =~ /^\w+$/) and $job_status)
{
  print_page("Invalid Result ID", qq(<p>This result ID is invalid. The corresponding result may have timed out and been deleted or you may have mis-typed your result ID. You may <a href="$CONFIG{'SERVERURL'}">submit a new query</a> to regenerate your result.</p>));
}
elsif ($job_status eq 'done')
{
  print $q->redirect("$CONFIG{'VIEWERURL'}?app=$CONFIG{'APPNAME'}&result_id=$result_id");
}
elsif ($job_status eq 'refused')
{
  print_page("Input $job_status", qq(<p>There was a problem processing your input. Make sure you have provided all the necessary information in the correct format. Please have a look at the <a href="$CONFIG{'HELPURL'}">help page</a> for more detailed instructions on how to submit queries to $CONFIG{'APPNAME'}.<br>If you think this is a bug, please email the author. Thanks.</p>\n<p>Status message:</p><pre>$status_message</pre>));
}
elsif ($job_status eq 'crashed')
{
  print_page("Job $job_status", qq(<p>The application crashed while processing your job. Make sure you have provided all the information in the correct format. If you're unsure what went wrong, you can check the <a href="$CONFIG{'VIEWERURL'}?view=outputlog&amp;result_id=$result_id">runtime log</a> for error messages. You may also want to have a look at the <a href="$CONFIG{'HELPURL'}">help page</a> for more detailed instructions on how to submit queries to $CONFIG{'APPNAME'}.<br>
  If you think this is a bug, please email the author. Thanks.\n</p><p>Status message:</p><pre>$status_message</pre>));
}
elsif ($job_status eq 'queued')
{
  print_page("Job $job_status", qq(<p>Your job has been submitted and is now in the queue, waiting to be processed. It has been queued for $t seconds now. Please wait for your job to be processed.<br>
  This page will automatically refresh every few seconds, and then forward you to your results when the job is done. You can safely reload this page as often as you like.<br>
  &nbsp;<br>
  If your job is still queued after several minutes, please send the author an email - the job processing service may not be running.<br>
  Should your job crash, try resubmitting your query, checking your input before you do. If this does not help, <a href="mailto:kelm\@stats.ox.ac.uk?subject=$CONFIG{'APPNAME'}%20web%20server%20problem">contact the author</a> within 24 hours and supply your Job ID and/or the exact data you submitted to $CONFIG{'APPNAME'}. Thanks.</p>), 10);
}
else
{
  my $fname = "$CONFIG{'CACHEDIR'}/$result_id/error.log.txt";
  my $logtxt = '';
  
  if (-f $fname)
  {
    open(my $stream, $fname) or finish("Could not open runtime log file for result '$result_id'.\n");
    local $/ = undef;
    $logtxt = <$stream>;
    close($stream);
  }
  
  print_page("Job $job_status", qq(<p>Your job has not finished running. It has been in its current state for $t seconds now. Please wait, while your job is running.<br>
  This page will automatically refresh every few seconds, and then forward you to your results when the job is done. You can safely reload this page as often as you like.<br/>
  &nbsp;</p>
  <p>--- Beginning of runtime log ---</p>
  <pre>$logtxt</pre>
  <p>--- End of runtime log ---</p>
  <p>&nbsp;<br/>
  If the logs show any alarming error messages and your job does not seem to finish even after 60 minutes (unless it is a very large job), try resubmitting your query, checking your input before you do. If this does not help, <a href="mailto:kelm\@stats.ox.ac.uk?subject=$CONFIG{'APPNAME'}%20web%20server%20problem">contact the author</a> and supply your JOB ID and/or the exact data you submitted to $CONFIG{'APPNAME'}. Thanks.\n</p><p>Status message:</p><pre>$status_message</pre>), 10);
}




#########################################################################



sub print_page
{
  my ($title, $explanation, $refresh) = @_;
  
  print $q->header('text/html');
  print
    qq(
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
                "http://www.w3.org/TR/html4/loose.dtd">
    <html>
    <head>
    <title>$CONFIG{'APPNAME'} : $title</title>
    );
  if (defined($refresh))
  {
    print
    qq(
    <meta http-equiv="refresh" content="$refresh; URL=).selflink('').qq(">
    );
  }
  print
    qq(
    <link rel="stylesheet" type="text/css" href="$CONFIG{'STYLESHEET'}">
    </head>
    <body>
    
    $CONFIG{'HTML_HEADER'}
    
    <p>&nbsp;</p>
    
    <h2>).html_pretty($title).qq(</h2>
    
    <form action=""><div>
    <p>
      Your Job ID is: <input class="copytext" type="text" size="50" value="$result_id" readonly>
    </p>
    </div></form>
    
    $explanation
    
    <p>&nbsp;</p>
    
    $CONFIG{'HTML_FOOTER'}
    
    </body>
    </html>
    );
  exit 0;
}
