#!/usr/bin/perl -T
#
# This CGI programme takes protein PDB codes/files or FASTA protein sequences as input.
#
# It gives back an annotated alignment between the input and CGDB membrane protein sequences.
# This can be in sequence or structure format.
#
# NOTE: This script runs in Perl's "Taint" mode ('-T' flag in first line).
#


use strict;
use Cwd  qw(abs_path);
use File::Basename;
use CGI;
use File::Glob ':glob';

my $q = new CGI;
my $EXECDIR = abs_path(dirname($0)); $EXECDIR =~ /^(.*)$/; $EXECDIR = $1;
require("$EXECDIR/cfg.pl");
my %CONFIG = %{get_config()};



###################################
#### SETTINGS AND PREPARATIONS ####
###################################


# Untaint environment variables
#
$ENV{'PATH'} = "/bin:/usr/bin:/usr/local/bin";
$ENV{'CDPATH'} = '.';
$ENV{'ENV'} = '';


my $app_subdir = '.';
if ($CONFIG{'APPNAME'} eq 'MEDELLER')
{
  $app_subdir = 'medeller';
}
elsif ($CONFIG{'APPNAME'} eq 'iMembrane')
{
  $app_subdir = 'imem';
}
elsif ($CONFIG{'APPNAME'} eq 'Completionist')
{
  $app_subdir = '.';
}


my $SELFURL     = $q->url();
my $result_id   = get_input('result_id', qr/^\w+$/);
my $view        = get_input('view', qr/^\w+$/, 'frameset');

my $modelprefix = ($CONFIG{'APPNAME'} eq 'MEDELLER' ? 'target_template.' : '');
my $modelext    = '.atm';

# Don't run anything, just print the results page corresponding to the $result_id.
#

if (($view eq 'header') or ($view eq 'footer') or ($view eq 'dummy'))
{
  print $q->header();
  print '
  <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
          "http://www.w3.org/TR/html4/loose.dtd">
  <html>
  <head>
  <title>'.$CONFIG{'APPNAME'}.'</title>
  <link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
  <base target="_parent">
  </head>
  <body>
  ';
  if ($view eq 'header')
  {
    print $CONFIG{'HTML_MINIMAL_HEADER'};
  }
  elsif ($view eq 'footer')
  {
    print $CONFIG{'HTML_MINIMAL_FOOTER'};
  }
  print '
  </body>
  </html>
  ';
}
#elsif (($view eq 'outputlog') or ($view eq 'submitlog'))
elsif ($view eq 'outputlog')
{
  #my $fname = ($view eq 'outputlog') ? "$CONFIG{'CACHEDIR'}/$result_id/error.log.txt" : "$CONFIG{'JOBSDIR'}/$result_id/submission.log.txt";
  my $fname = "$CONFIG{'CACHEDIR'}/$result_id/error.log.txt";
  
  if (-f $fname)
  {
    open(my $stream, $fname) or finish("Could not open '$view' log file for result '$result_id'.\n");
    local $/ = undef;
    my $text = <$stream>;
    close($stream);
    print $q->header('text/plain');
    print $text;
  }
  else
  {
    print $q->header('text/plain');
    print "(log file not found)\n";
    if (($view eq 'outputlog') and (-d "$CONFIG{'JOBSDIR'}/$result_id"))
    {
      print "(Your job may not have run yet, so there won't be any runtime log file yet.)\n";
    }
  }
}
elsif (!length($result_id) or !(-d "$CONFIG{'CACHEDIR'}/$result_id"))
{
  print $q->header('text/html');
  print
  '
  <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
              "http://www.w3.org/TR/html4/loose.dtd">
  <html>
  <head>
  <title>'.$CONFIG{'APPNAME'}.': Results not found</title>
  <link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
  </head>
  <body>
  '.$CONFIG{'HTML_HEADER'}.'
  
  <p>&nbsp;</p>
  
  <h2>'.html_pretty('Results not found').'</h2>
  
  <p>
  The result ID you requested was: &quot;'.$result_id.'&quot
  </p>
  <p>
  This result ID is invalid. The corresponding result may have timed out and been deleted or you may have mis-typed your result ID.
  </p>
  
  <p>
  You may <a href="'.$CONFIG{'SERVERURL'}.'">submit a new query</a> to regenerate your result.
  </p>
  
  <p>&nbsp;</p>
  
  '.$CONFIG{'HTML_FOOTER'}.'
  
  </body>
  </html>
  ';
}
else
{
  chdir("$CONFIG{'CACHEDIR'}/$result_id");
  
  #my $view     = get_input('view', qr/^\w+$/, 'frameset'); # line moved up next to $result_id check
  my $dataid   = get_input('dataid', qr/^$|^[\w\.\-,]+$/, '');
  my $type     = get_input('type', qr/^$|^contact$|^layer$|^confidence$/, '');
  my $viewmode = get_input('viewmode', qr/^$|^\w+$/, '');
  
  if ($view eq 'frameset')
  {
    print $q->header('text/html');
    
    my $havehits = 0;
    
    if ($CONFIG{'APPNAME'} eq 'MEDELLER')
    {
      if (-f "$app_subdir/${modelprefix}core${modelext}")
      {
        $havehits = 1;
      }
    }
    elsif ($CONFIG{'APPNAME'} eq 'iMembrane')
    {
      open(my $hit_file, 'imem/hits.table');
      local $/ = undef;
      my $hits_list = <$hit_file>;
      close($hit_file);
      
      if (length($hits_list) >= 2)
      {
        $havehits = 1;
      }
    }
    elsif ($CONFIG{'APPNAME'} eq 'Completionist')
    {
      if (-f "$app_subdir/${modelprefix}hiacc${modelext}")
      {
        $havehits = 1;
      }
    }
    
    unless ($havehits)
    {
          print
          '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
          "http://www.w3.org/TR/html4/loose.dtd">
          <html>
          <head>
          <title>'.$CONFIG{'APPNAME'}.': Results not found</title>
          <link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
          </head>
          <body>
          '.$CONFIG{'HTML_HEADER'}.'

          <p>&nbsp;</p>
          
          ';
          
          
          if ($CONFIG{'APPNAME'} eq 'MEDELLER')
          {
            print '
              <h2>'.html_pretty('No matches').'</h2>
              <p>Your search query did not produce a model.</p>';
          }
          elsif ($CONFIG{'APPNAME'} eq 'iMembrane')
          {
            print '
              <h2>'.html_pretty('No matches').'</h2>
              <p>Your search query returned no hits.</p>';
          }
          elsif ($CONFIG{'APPNAME'} eq 'Completionist')
          {
            print '
              <h2>'.html_pretty('Failure').'</h2>
              <p>Failed to complete your model.</p>';
          }
          
          print '
          <p>
          The result ID you requested was: &quot;'.$result_id.'&quot;
          </p>';
          
          if ($CONFIG{'APPNAME'} eq 'MEDELLER' or $CONFIG{'APPNAME'} eq 'iMembrane')
          {
            print '
            <p>
            Try <a href="'.$CONFIG{'SERVERURL'}.'">resubmitting your query</a> with a less stringent E-value / sequence score / structure score <strong>cut-off</strong>.
            </p>
            
            ';
          }
          elsif ($CONFIG{'APPNAME'} eq 'Completionist')
          {
            print '
            <p>
            This could be a problem with your input or a bug. Try <a href="'.$CONFIG{'SERVERURL'}.'">resubmitting your query</a>. If that does not help, please <a href="mailto:kelm@stats.ox.ac.uk?subject='.$CONFIG{'APPNAME'}.'%20web%20server%20problem">contact the author</a>.
            </p>
            
            ';
          }
          
          if (-f "$CONFIG{'CACHEDIR'}/$result_id/error.log.txt")
          {
            print '<p>You can also <a href="'.selflink('view=outputlog').'">view the error log</a> to check that this is not a <strong>bug</strong>. If you think it is, please <a href="mailto:kelm@stats.ox.ac.uk?subject='.$CONFIG{'APPNAME'}.'%20web%20server%20problem">report it to the author</a> and mention your job ID.</p>';
          }
          print '
          <p>&nbsp;</p>

          '.$CONFIG{'HTML_FOOTER'}.'
          </body>
          </html>
          ';
          exit 0;
    }
    
    print
    '
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN"
   "http://www.w3.org/TR/html4/frameset.dtd">
    <html>
    <head>
    <title>'.$CONFIG{'APPNAME'}.': Results</title>
    </head>';
    
    
    if ($CONFIG{'APPNAME'} eq 'MEDELLER' or $CONFIG{'APPNAME'} eq 'iMembrane')
    {
      print '
    <frameset cols="20,*,20" framespacing="0" frameborder="0" border="0">
      <frame src="'.selflink('view=dummy').'" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
      <frameset rows="50,300,*,30" framespacing="0" frameborder="0" border="0">
        
        <frame src="'.selflink('view=header').'" name="Header" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
        
        <frameset id="viewerframeset" cols="300,300,*" framespacing="0" frameborder="0" border="0">
          <frame src="'.selflink('view=dummy').'" name="StructureViewerLeft" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
          <frame src="'.selflink('view=dummy').'" name="StructureViewerRight" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
          <frame src="'.selflink('view=dummy').'" name="SequenceViewer" frameborder="0" marginwidth="0" marginheight="0" noresize>
        </frameset>
        
        <frame src="'.selflink('view=list').'" name="ResultList" frameborder="0" marginwidth="0" marginheight="0" noresize>
        
        <frame src="'.selflink('view=footer').'" name="Footer" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
      </frameset>
      <frame src="'.selflink('view=dummy').'" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>';
    }
    elsif ($CONFIG{'APPNAME'} eq 'Completionist')
    {
      print '
    <frameset cols="20,*,20" framespacing="0" frameborder="0" border="0">
      <frame src="'.selflink('view=dummy').'" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
      <frameset rows="50,*,30" framespacing="0" frameborder="0" border="0">
        
        <frame src="'.selflink('view=header').'" name="Header" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
      
        <frameset id="viewerframeset" cols="600,*" framespacing="0" frameborder="0" border="0">
          <frame src="'.selflink('view=dummy').'" name="StructureViewerLeft" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
          <frame src="'.selflink('view=list').'" name="ResultList" frameborder="0" marginwidth="0" marginheight="0" noresize>
        </frameset>
        
        <frame src="'.selflink('view=footer').'" name="Footer" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
      </frameset>
      <frame src="'.selflink('view=dummy').'" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>';
    }
    
    print '
      
      <noframes>
      <body>
      '.$CONFIG{'HTML_HEADER'}.'
      <p>
        Your result ID is: <a href="'.selflink('').'">'.$result_id.'</a><br>
        Your browser does not support frames. You may still view the list of results by clicking on the above result id.<br>
        For optimal viewing, please use a browser that does support frames.
      </p>
      '.$CONFIG{'HTML_FOOTER'}.'
      </body>
      </noframes>
    
    </frameset>
    </html>
    ';
  }
  elsif ($view eq 'structure')
  {
    my $jmolscript;
    
    unless ($dataid)
    {
      finish("Missing dataid (model type)\n");
    }
    unless ($type)
    {
      finish("Must supply an annotation type ('contact', 'layer' or 'confidence').\n");
    }
    
    my $structure_title = '';
    my $modelpath = '';
    my $dimensions = '300, 280';

    
    if ($CONFIG{'APPNAME'} eq 'MEDELLER' or $CONFIG{'APPNAME'} eq 'Completionist')
    {
      $structure_title = $type;
      if ($type eq 'confidence')
      {
        $modelpath = "$result_id/$app_subdir/${modelprefix}$dataid${modelext}";
        $jmolscript = 'wireframe off; spacefill off; rotate X -90; cartoon on; color Temperature;';
      }
      else
      {
        $modelpath = "$result_id/$app_subdir/$dataid/$type${modelext}";
        $jmolscript = 'wireframe off; spacefill off; rotate X -90; cartoon on; color Temperature;';
      }
    }
    elsif ($CONFIG{'APPNAME'} eq 'iMembrane')
    {
      $structure_title = $viewmode;
      if ($viewmode eq 'match')
      {
        $modelpath = "$result_id/imem/$dataid/parent/$type${modelext}";
        #$jmolscript = 'wireframe off; spacefill on; rotate X -90; color Temperature;';
        $jmolscript = 'wireframe off; spacefill off; rotate X -90; cartoon on; color Temperature;';
      }
      else
      {
        $modelpath = "$result_id/imem/$dataid/$type${modelext}";
        $jmolscript = 'wireframe off; spacefill off; rotate X -90; cartoon on; color Temperature;';
      }
    }
    
    if ($CONFIG{'APPNAME'} eq 'Completionist')
    {
      $dimensions = '600, 560';
    }
    
    print $q->header('text/html');
    print
    '
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
                "http://www.w3.org/TR/html4/loose.dtd">
    <html>
    <head>
    <title>'.$CONFIG{'APPNAME'}.': Results</title>
    <link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
    <script type="text/javascript" src="jmol/Jmol.js"></script>
    </head>
    <body style="background-color:#FFFFFF;">
      <script type="text/javascript">
        // <!--
        jmolInitialize("jmol", true);
        jmolSetAppletColor("white");
        // -->
      </script>
      <h3 class="subframeheader">'.html_pretty($structure_title, 1).'</h3>
      <form action="">
        <script type="text/javascript">
          // <!--
          jmolApplet(['.$dimensions.'], "load '.$CONFIG{'CACHERELATIVEURL'}.'/'.$modelpath.'; '.$jmolscript.'");
          jmolBr();
          // -->
        </script>
      </form>
    
    </body>
    </html>
    ';
  }
  elsif ($view eq 'sequence')
  {
    
    unless ($dataid and (-d "$app_subdir/$dataid"))
    {
      finish("Illegal dataid: $dataid\n");
    }
    
    open(my $f, "<$app_subdir/$dataid/annotation.inc.html") or finish("Could not open sequence annotation file '$dataid'\n");
    local $/ = undef;
    my $text = <$f>;
    close($f);

    print $q->header('text/html');
    print
    '
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
                "http://www.w3.org/TR/html4/loose.dtd">
    <html>
    <head>
    <title>'.$CONFIG{'APPNAME'}.': Results</title>
    <link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
    <link rel="stylesheet" type="text/css" href="seqalign.css">
    </head>
    <body>
      <h3 class="subframeheader">'.html_pretty('Sequence Alignment').'</h3>
      <p>&nbsp;</p>
      <table cellpadding="0" cellspacing="0" border="0" class="seqalign">
      '.$text.'
      </table>
      <p>&nbsp;</p>
      <p>&nbsp;</p>
      <p>&nbsp;</p>
      <div class="colorkey-outer">
        <h4>Colour Key</h4>
        <div class="colorkey-inner">
        <p class="membrane_T">T: Inner membrane (lipid <strong>T</strong>ails)</p>
        <p class="membrane_H">H: Outer membrane (polar <strong>H</strong>ead groups)</p>
        <p class="membrane_N">N: <strong>N</strong>on-membrane</p>
        </div>
      </div>
    </body>
    </html>
    ';
  }
  elsif ($view eq 'list')
  {
    my $show_structures = 0;
    my $cgdblink = 'http://sbcb.bioch.ox.ac.uk/cgdb/simtable.php?pdb=';
    my @hit_table;
    my @columns;
    
    open(my $hitfile, 'imem/hits.table') or finish("Error opening hits.table");
    @hit_table = <$hitfile>;
    close($hitfile);
    
    my $temp = shift @hit_table;
    $temp =~ s/^#//;
    $temp =~ s/dbid/iMemID/;
    @columns = split(/\t/ ,$temp);
    
    # Do we have structure output?
    # Check if we should switch off structure viewer...
    #
    my $rmsd_col = -1;
    for (my $i=0; $i<@columns; $i++)
    {
      if ($columns[$i] eq 'rmsd')
      {
        $rmsd_col = $i;
        last;
      }
    }
    if (int(@hit_table))
    {
      my @fields = split(/\t/, $hit_table[0]);
      if ($fields[$rmsd_col] >= 0)
      {
        $show_structures = 1;
      }
    }
    
    # DEBUG : Switch off structure viewer
    #$show_structures = 0;
    
    
    print $q->header('text/html');
    print
    '
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
                "http://www.w3.org/TR/html4/loose.dtd">
    <html>
    <head>
    <title>'.$CONFIG{'APPNAME'}.': Results</title>
    <link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
    <base target="_parent">
    </head>
    <body>
    
    <div class="mainmargin">
    ';
    
    unless ($show_structures)
    {
      print q{
        <script type="text/javascript">
          // <!--
          parent.document.getElementById('viewerframeset').cols = '0,300,*';
          // -->
        </script>
      };
    }
    
    print '<h2>'.html_pretty('Results').'</h2>';
    print qq(
    <p>
      Your result ID is: <code class="copytext">$result_id</code>
    </p>
    <p>
      You can keep a note of the above result ID or save a <a href=").selflink('').qq(">link to this page</a>, in case you need to come back to your results within 24 hours. After this time your results will be deleted from our server and you will need to re-submit your query.
    </p>
    <p>
      If you need help reading these results, have a look at the <a target="_blank" href="$CONFIG{'HELPURL'}#viewresults">Help</a> page. If you still have questions after that, please feel free to <a href="mailto:kelm\@stats.ox.ac.uk?subject=$CONFIG{'APPNAME'}%20web%20server%20output">contact the author</a>.
    </p>
    <p>&nbsp;</p>
    );
    
    my $url_strucA = '';
    my $url_strucB = '';
    my $url_align = '';
    
    print '
    <p>
    Every hit that your query generated is represented as a single row in the table below.<br>
    Click on the "Layer" or "Contact" links to display the corrresponding information for that hit.
    </p>
    ';
    
    print '
      <table cellpadding="0" cellspacing="0" border="0" class="resultlist">
      <tr>
    ';
    foreach my $col (@columns[2..$#columns])
    {
      my @hfields = split(/[\s\-\_]+/, $col, 2);
      my $h = lc($hfields[0]);
      print qq(<th><a target="_blank" href="$CONFIG{'HELPURL'}#column_$h">$col</a></th>);
    }
    print qq(<th colspan="2"><a target="_blank" href="$CONFIG{'HELPURL'}#column_view">View Results</a></th><th><a target="_blank" href="$CONFIG{'HELPURL'}#column_download">Download Results</a></th>);
    print '</tr>';
    
    
    my $structure_loaded = 0;
    foreach (my $i=0; $i<@hit_table; $i++)
    {
      my @hit = split(/\t/, $hit_table[$i]);
      $hit[2] = qq(<a target="_blank" href="$SELFURL?app=iMemDB&amp;dataid=$hit[2]">$hit[2]</a>);
      my $hitid = $hit[0];
      
      my $shade = ($i % 2) + 1;
      
      print '<tr class="shade'.$shade.'"><td>';
      print join('</td><td>', @hit[2..$#hit]);
      print '</td>';
      
      my $url_query_layer   = selflink("view=structure&amp;viewmode=query&amp;dataid=$hitid&amp;type=layer");
      my $url_match_layer   = selflink("view=structure&amp;viewmode=match&amp;dataid=$hitid&amp;type=layer");
      my $url_query_contact = selflink("view=structure&amp;viewmode=query&amp;dataid=$hitid&amp;type=contact");
      my $url_match_contact = selflink("view=structure&amp;viewmode=match&amp;dataid=$hitid&amp;type=contact");
      
      unless ($structure_loaded)
      {
        $structure_loaded = 1;
        $url_strucA = $url_query_layer;
        $url_strucB = $url_match_layer;
        $url_align  = selflink("view=sequence&amp;dataid=$hitid&amp;type=layer");
      }
      
      if ($show_structures)
      {
        print qq(<td><a target="SequenceViewer" href=").selflink("view=sequence&amp;dataid=$hitid").qq(" onclick="parent['StructureViewerLeft'].location.href = '$url_query_layer';parent['StructureViewerRight'].location.href = '$url_match_layer'; return true;">Layer</a></td>);
        print qq(<td><a target="SequenceViewer" href=").selflink("view=sequence&amp;dataid=$hitid").qq(" onclick="parent['StructureViewerLeft'].location.href = '$url_query_contact';parent['StructureViewerRight'].location.href = '$url_match_contact'; return true;">Contact</a></td>);
        # print qq(<td><span class="brokenlink">Contact</span></td>);
      }
      else
      {
        print qq(<td><a target="SequenceViewer" href=").selflink("view=sequence&amp;dataid=$hitid").qq(" onclick="parent['StructureViewerRight'].location.href = '$url_match_layer'; return true;">Layer</a></td>);
        print qq(<td><a target="SequenceViewer" href=").selflink("view=sequence&amp;dataid=$hitid").qq(" onclick="parent['StructureViewerRight'].location.href = '$url_match_contact'; return true;">Contact</a></td>);
        #print qq(<td><a target="SequenceViewer" href=").selflink("view=sequence&amp;dataid=$hitid").qq(">Layer</a></td>);
        #print qq(<td><a target="SequenceViewer" href=").selflink("view=sequence&amp;dataid=$hitid").qq(">Contact</a></td>);
        # print qq(<td><span class="brokenlink">Contact</span></td>);
      }
      
      print qq(<td><a href=").selflink("view=download&amp;dataid=$hitid", "iMembrane_hit_$hitid.tar.gz").qq(">Download hit (tar/gz archive)</a></td>);
      # print qq(<td><span class="brokenlink">Download hit (tar/gz archive)</span></td>);
      
      print "</tr>\n";
    }
    
    print '
    </table>
    ';
    
    
    if ($url_align)
    {
      print qq{
        <script type="text/javascript">
          // <!--
          parent['SequenceViewer'].location.href = "$url_align";
          // -->
        </script>
      };
    }
    
    if ($show_structures and $url_strucA and $url_strucB)
    {
      print qq{
        <script type="text/javascript">
          // <!--
          parent['StructureViewerLeft'].location.href = '$url_strucA';
          parent['StructureViewerRight'].location.href = '$url_strucB';
          // -->
        </script>
      };
    }
    elsif ($url_strucB)
    {
      print qq{
        <script type="text/javascript">
          // <!--
          parent['StructureViewerRight'].location.href = '$url_strucB';
          // -->
        </script>
      };
    }
    elsif ($url_strucA)
    {
      print qq{
        <script type="text/javascript">
          // <!--
          parent['StructureViewerLeft'].location.href = '$url_strucA';
          // -->
        </script>
      };
    }
    
    
    print qq(
    <p>&nbsp;</p>
    <p>
    You can now:
    </p>
    <ul>
    <li>
    <a href=").selflink('view=download&amp;dataid=all', $CONFIG{'APPNAME'}.'_all.tar.gz').qq(">Download all the above results (tar/gz archive)</a>
    </li>
    );
    
    if ($CONFIG{'APPNAME'} eq 'MEDELLER')
    {
      # MEDELLER always runs iMembrane, so we always want a link to the search results.
      print qq(
      <li>
      <a target="_blank" href="$SELFURL?app=iMembrane&amp;result_id=$result_id">View the results of the iMembrane search, whose top hit was used to annotate your template</a>
      </li>
      );
    }
    elsif ($CONFIG{'APPNAME'} eq 'iMembrane')
    {
      if (-d 'medeller')
      {
        # We have generated a MEDELLER model. We need a link to go see it.
        print qq(
        <li>
        <a target="_blank" href="$SELFURL?app=MEDELLER&amp;result_id=$result_id">See your MEDELLER model</a>
        </li>
        );
      }
    }
    
    print qq(
    <li>
    <a target="_blank" href="$CONFIG{'CACHEURL'}/$result_id/$app_subdir">View the complete list of output files in your browser</a>
    </li>);
    
    if ($CONFIG{'APPNAME'} ne 'Completionist')
    {
      print '
    <li>
    <a target="_blank" href="'.selflink('view=outputlog').'">View the runtime log file</a>
    </li>';
    }
    
    print '
    </ul>
    
    <p class="note">Note: the files ending in &quot;.atm&quot; contain the PDB co-ordinates of your models.</p>

    </div>
    
    </body>
    </html>'."\n";
    
  }
  elsif ($view eq 'download')
  {
    if ($dataid eq 'all')
    {
      $dataid = '';
    }
    if (! (-d "$app_subdir/$dataid"))
    {
      finish("Data ID not valid: $dataid (app_subdir: $app_subdir)\n");
    }
    
    open(my $stream, "tar zcf - $app_subdir/$dataid |") or finish("Could not open tar/gz stream for downloading.\n");
    print $q->header('application/x-Gzip');
    my $buffer;
    while (my $bytesread=read($stream,$buffer,1024))
    {
      print $buffer;
    }
    close($stream);
  }
  else
  {
    finish("Illegal view type: $view\n");
  }
  
  exit 0;
}



#########################################################################


sub finish
{
  my ($message) = @_;
  if (!defined($message))
  {
    $message = '(no crash message available)';
  }
  
  print $q->header('text/html');
  print qq(
  <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
              "http://www.w3.org/TR/html4/loose.dtd">
  <html>
    <head>
    <title>).$CONFIG{'APPNAME'}.qq(: Error</title>
    <link rel="stylesheet" type="text/css" href=").$CONFIG{'STYLESHEET'}.qq(">
    </head>
    <body>
    $CONFIG{'HTML_HEADER'}
    
    <p>&nbsp;</p>
    
    <h2>).html_pretty('Error').qq(</h2>
    
    <p>
    <em>$message</em>
    </p>
   
    <p>&nbsp;</p> 
    
    <p>
    The application encountered a problem and quit with the above error message.<br>
    If this occurred because you typed in a URL manually, please DO NOT<br>
    contact the author.<br>
    If you got here via the ).$CONFIG{'APPNAME'}.qq( front page and this keeps<br>
    occurring, even though you seem to be doing everything right,<br>
    please DO <a href="mailto:kelm\@stats.ox.ac.uk?subject=$CONFIG{'APPNAME'}%20web%20server%20problem">contact the author</a> and include your resultID, the above error<br>
    message and an exact description of what you did to get here.
    </p>
    
    <p>
    For now, you may try <a href="$CONFIG{'SERVERURL'}">submitting a new query</a>.
    </p>
    
    <p>&nbsp;</p>
    
    
    $CONFIG{'HTML_FOOTER'}
    </body>
  </html>
  \n);
  exit 0;
}
