#!/usr/bin/perl -T
#
# This CGI programme takes protein PDB codes/files or FASTA protein sequences as input.
#
# It gives back an annotated alignment between the input and CGDB membrane protein sequences.
# This can be in sequence or structure format.
#
# NOTE: This script runs in Perl's "Taint" mode ('-T' flag in first line).
#


use strict;
use Cwd  qw(abs_path);
use File::Basename;
use CGI;
use File::Glob ':glob';

my $q = new CGI;
my $EXECDIR = abs_path(dirname($0)); $EXECDIR =~ /^(.*)$/; $EXECDIR = $1;
require("$EXECDIR/cfg.pl");
my %CONFIG = %{get_config()};



###################################
#### SETTINGS AND PREPARATIONS ####
###################################


# Untaint environment variables
#
$ENV{'PATH'} = "/bin:/usr/bin:/usr/local/bin";
$ENV{'CDPATH'} = '.';
$ENV{'ENV'} = '';


my $SELFURL = $q->url();
#my $result_id = get_input('result_id', qr/^$|^imemdb$/, "imemdb");
my $result_id = 'imemdb';
my $dataid = get_input('dataid', qr/^$|^\w{4}(.\d+(.\w(.\d+(.\d+)?)?)?)?$/, "");

$dataid =~ /(\w{4})(?:\.(\d+)(?:\.(\w+)(?:\.(\d+)(?:\.(\d+))?)?)?)?/;
my $pdbid = "$1";
my $simulationid = "$2";
my $proteinid = "$1.$2";
my $chainid = "$3";
my $matchid = "$1.$2.$3.$4";
my $dataid_is_complete = ($dataid =~ /^\w{4}.\d+.\w.\d+.\d+$/);
my $dataid_has_simulation = ($dataid =~ /^\w{4}.\d+/);

my $cgdblink = 'http://sbcb.bioch.ox.ac.uk/cgdb/simtable.php?pdb=';
my $pdblink = 'http://www.pdb.org/pdb/explore/explore.do?structureId=';


my $view        = get_input('view', qr/^\w+$/, 'frameset');

my $modelext    = '.atm';
my $min_representative_tm_score = 0.5;


#my $db = '/Users/seb/webserver/databases/imemdb/current/entries';
my $db = $CONFIG{'CACHEDIR'} . '/' . $result_id . '/entries';


if (($view eq 'header') or ($view eq 'footer') or ($view eq 'dummy'))
{
  print $q->header();
  print '
  <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
          "http://www.w3.org/TR/html4/loose.dtd">
  <html>
  <head>
  <title>'.$CONFIG{'APPNAME'}.'</title>
  <link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
  <base target="_parent">
  </head>
  <body>
  ';
  if ($view eq 'header')
  {
    print $CONFIG{'HTML_MINIMAL_HEADER'};
  }
  elsif ($view eq 'footer')
  {
    print $CONFIG{'HTML_MINIMAL_FOOTER'};
  }
  print '
  </body>
  </html>
  ';
}
else
{
  chdir("$db");
  
  my $type     = get_input('type', qr/^$|^contact$|^layer$|^confidence$/, '');
  my $viewmode = get_input('viewmode', qr/^$|^\w+$/, '');
  
  if ($view eq 'frameset')
  {
    print $q->header('text/html');
    
    print
    '
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN"
   "http://www.w3.org/TR/html4/frameset.dtd">
    <html>
    <head>
    <title>'.$CONFIG{'APPNAME'}.'</title>
    </head>';
    
    
    print '
  <frameset cols="20,*,20" framespacing="0" frameborder="0" border="0">
    <frame src="'.selflink('view=dummy').'" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
    <frameset rows="50,300,*,30" framespacing="0" frameborder="0" border="0">
      
      <frame src="'.selflink('view=header').'" name="Header" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
      
      <frameset id="viewerframeset" cols="300,300,*" framespacing="0" frameborder="0" border="0">
        <frame src="'.selflink('view=dummy').'" name="StructureViewerLeft" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
        <frame src="'.selflink('view=dummy').'" name="StructureViewerRight" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
        <frame src="'.selflink('view=dummy').'" name="SequenceViewer" frameborder="0" marginwidth="0" marginheight="0" noresize>
      </frameset>
      
      <frameset id="viewerframeset" cols="260,*" framespacing="0" frameborder="0" border="0">
        <frame src="'.selflink("view=index&amp;dataid=$dataid").'" name="Index" frameborder="0" marginwidth="0" marginheight="0" noresize>
        <frame src="'.selflink("view=dummy").'" name="ResultList" frameborder="0" marginwidth="0" marginheight="0" noresize>
      </frameset>
      
      <frame src="'.selflink('view=footer').'" name="Footer" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>
    </frameset>
    <frame src="'.selflink('view=dummy').'" scrolling="no" frameborder="0" marginwidth="0" marginheight="0" noresize>';
    
    print '
      
      <noframes>
      <body>
      '.$CONFIG{'HTML_HEADER'}.'
      <p>
        Your browser does not support frames. You may still view the list of results by clicking on the above result id.<br>
        For optimal viewing, please use a browser that does support frames.
      </p>
      '.$CONFIG{'HTML_FOOTER'}.'
      </body>
      </noframes>
    
    </frameset>
    </html>
    ';
  }
  elsif ($view eq 'structure')
  {
    my $jmolscript;
    
    unless ($type)
    {
      finish("Must supply an annotation type ('contact', 'layer' or 'confidence').\n");
    }
    
    my $structure_title = '';
    my $modelpath = '';
    my $dimensions = '300, 280';

    
    if ($viewmode eq 'match')
    {
      $structure_title = "CGDB snapshot: $type";
      $modelpath = "$result_id/entries/$dataid/parent/$type${modelext}";
      $jmolscript = 'hide *; display *.CA; wireframe off; spacefill on; rotate X -90; color Temperature;';
      #$jmolscript = 'wireframe off; spacefill off; rotate X -90; cartoon on; color Temperature;';
    }
    else
    {
      $structure_title = "PDB structure: $type";
      $modelpath = "$result_id/entries/$dataid/$type${modelext}";
      $jmolscript = 'wireframe off; spacefill off; rotate X -90; cartoon on; color Temperature;';
    }
    
    print $q->header('text/html');
    print
    '
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
                "http://www.w3.org/TR/html4/loose.dtd">
    <html>
    <head>
    <title>'.$CONFIG{'APPNAME'}.': Results</title>
    <link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
    <script type="text/javascript" src="jmol/Jmol.js"></script>
    </head>
    <body style="background-color:#FFFFFF;">
      <script type="text/javascript">
        // <!--
        jmolInitialize("jmol", true);
        jmolSetAppletColor("white");
        // -->
      </script>
      <h3 class="subframeheader">'.html_pretty($structure_title, 1).'</h3>
      <form action="">
        <script type="text/javascript">
          // <!--
          jmolApplet(['.$dimensions.'], "load '.$CONFIG{'CACHERELATIVEURL'}.'/'.$modelpath.'; '.$jmolscript.'");
          jmolBr();
          // -->
        </script>
      </form>
    
    </body>
    </html>
    ';
  }
  elsif ($view eq 'sequence')
  {
    open(my $f, '<', "$db/$dataid/annotation.inc.html") or finish("Could not open sequence annotation file\n");
    my @lines = <$f>;
    close($f);
    @lines = grep(/>(structure|membrane layer|membrane contact)</, @lines);
    my $text = join('', @lines);
    $text =~ s/>$dataid</>PDB structure</g;
    $text =~ s/>$matchid</>CGDB snapshot</g;
    $text =~ s/>structure</>sequence</g;

    print $q->header('text/html');
    print
    '
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
                "http://www.w3.org/TR/html4/loose.dtd">
    <html>
    <head>
    <title>'.$CONFIG{'APPNAME'}.': Results</title>
    <link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
    <link rel="stylesheet" type="text/css" href="seqalign.css">
    </head>
    <body>
      <h3 class="subframeheader">'.html_pretty("Structure Alignment &nbsp; ($dataid)").'</h3>
      <p>&nbsp;</p>
      <table cellpadding="0" cellspacing="0" border="0" class="seqalign">
      '.$text.'
      </table>
      <p>&nbsp;</p>
      <p>&nbsp;</p>
      <p>&nbsp;</p>
      <div class="colorkey-outer">
        <h4>Colour Key</h4>
        <div class="colorkey-inner">
        <p class="membrane_T">T: Inner membrane (lipid <strong>T</strong>ails)</p>
        <p class="membrane_H">H: Outer membrane (polar <strong>H</strong>ead groups)</p>
        <p class="membrane_N">N: <strong>N</strong>on-membrane</p>
        </div>
      </div>
    </body>
    </html>
    ';
  }
  elsif ($view eq 'index')
  {
    my $show_structures = 0;
    my @hit_table;
    my @columns;
    
    open(my $hitfile, '<', "$db/../representatives.table") or finish("Error opening representatives.table");
    @hit_table = <$hitfile>;
    close($hitfile);
    
    @columns = ("Protein", "Crosslinks", "View &nbsp;&nbsp;");
    
    print $q->header('text/html');
    print
    '
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
                "http://www.w3.org/TR/html4/loose.dtd">
    <html>
    <head>
    <title>'.$CONFIG{'APPNAME'}.': iMembrane Database Index</title>
    <link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
    <base target="_parent">
    </head>
    <body>
    
    <div class="mainmargin">
    ';
    
    print '<h2>'.html_pretty('All Proteins').'</h2>';
    print '
      <p>Select a protein to view.</p>
      <table cellpadding="0" cellspacing="0" border="0" class="resultlist">
      <tr>
    ';
    foreach my $col (@columns)
    {
      my @hfields = split(/[\s\-\_]+/, $col, 2);
      my $h = lc($hfields[0]);
      print qq(<th><a target="_blank" href="$CONFIG{'HELPURL'}#column_$h">$col</a></th>);
    }
    print '</tr>';
    
    my $url_list_default = '';
    
    foreach (my $i=0; $i<@hit_table; $i++)
    {
      my @hit = split(/\t/, $hit_table[$i]);
      my $shade = ($i % 2) + 1;
      
      if ($hit[2] < $min_representative_tm_score)
      {
        next;
      }
      
      my $hitid = $hit[1];
      my $url_list = selflink("view=list&amp;dataid=$hitid&amp;type=layer");
      unless ($url_list_default)
      {
        if (($dataid_has_simulation and $proteinid eq $hitid) or (!$dataid_has_simulation and $pdbid eq $hit[0]) or !$dataid)
        {
          $url_list_default = $url_list;
        }
      }
      
      print qq(
      <tr class="shade$shade">
        <td>
          $hit[0]
        </td>
        <td>
          <a target="_blank" href="$pdblink$hit[0]">PDB</a>
          <a target="_blank" href="$cgdblink$hit[0]">CGDB</a>
        </td>
        <td>
          <a target="ResultList" href="$url_list">View &gt;</a>
        </td>
      </tr>
      );
    }
    
    if ($url_list_default)
    {
      print qq(
      <script type="text/javascript">
        // <!--
        parent['ResultList'].location.href = "$url_list_default";
        // -->
      </script>
      );
    }
    
    print qq(
    </table>
    </div>
    
    </body>
    </html>\n);
    
  }
  elsif ($view eq 'list')
  {
    my $show_structures = 0;
    my @columns;
    
    my $tmscore = '';
    {
      open(my $hitfile, '<', "$db/../representatives.table") or finish("Error opening representatives.table");
      my @hit_table = <$hitfile>;
      close($hitfile);
      my @hits = grep(/^$pdbid\s/, @hit_table);
      if (@hits)
      {
        my @fields = split(/\t/, $hits[0]);
        $tmscore = sprintf("%.2f", @fields[2]);
      }
    }
    
    my @chains = sort(glob("$db/$proteinid.*.*.*"));
    
    my $transmembrane_chain = '';
    if (@chains == 1)
    {
      $transmembrane_chain = basename($chains[0]);
    }
    elsif (!$dataid_is_complete)
    {
      my $num_tail_residues = -1;
      
      foreach my $chain (@chains)
      {
        open(my $f, '<', "$chain/query_annotation.tem") or finish("Could not open sequence annotation file\n");
        my @entries;
        {
          local $/ = "*\n";
          @entries = <$f>;
        }
        close($f);
        
        @entries = grep(/\nmembrane contact\n/, @entries);
        my @fields = split(/\n/, $entries[0]);
        my $count = 0;
        while ( $fields[2] =~ s/T// )
        {
           ++$count;
        }
        
        if ($count > $num_tail_residues)
        {
          $num_tail_residues = $count;
          $transmembrane_chain = basename($chain);
        }
      }
    }
    
    
    
    @columns = qw(Protein Chain iMemID);
    
    print $q->header('text/html');
    print
    '
    <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
                "http://www.w3.org/TR/html4/loose.dtd">
    <html>
    <head>
    <title>'.$CONFIG{'APPNAME'}.': Results</title>
    <link rel="stylesheet" type="text/css" href="'.$CONFIG{'STYLESHEET'}.'">
    <base target="_parent">
    </head>
    <body>
    
    <div class="mainmargin">
    ';
    
    print '<h2>'.html_pretty("Protein $pdbid chains").'</h2>';
    
    my $url_strucA = '';
    my $url_strucB = '';
    my $url_align = '';
    
    print qq(
    <p>
      Based on simulation $simulationid of protein <a target="_blank" href="$pdblink$pdbid">$pdbid</a>, as archived in the <a target="_blank" href="$cgdblink$pdbid">CGDB database</a>.<br>
      Structural agreement between final CGDB snapshot and PDB-deposited structure is $tmscore (TM-score averaged over all chains).
    </p>
    <p>
      Select a chain to view.
    </p>
    );
    
    print '
      <table cellpadding="0" cellspacing="0" border="0" class="resultlist">
      <tr>';
    foreach my $col (@columns)
    {
      my @hfields = split(/[\s\-\_]+/, $col, 2);
      my $h = lc($hfields[0]);
      print qq(
        <th><a target="_blank" href="$CONFIG{'HELPURL'}#column_$h">$col</a></th>);
    }
    print qq(
        <th><a target="_blank" href="$CONFIG{'HELPURL'}#column_view">View Annotation</a></th><th><a target="_blank" href="$CONFIG{'HELPURL'}#column_download">Download Annotation</a></th>
      </tr>);
    
    my $structure_loaded = 0;
    for (my $i=0; $i<@chains; ++$i)
    {
      my $shade = ($i % 2) + 1;
      my $hitid = basename($chains[$i]);
      
      my $url_query_layer   = selflink("view=structure&amp;viewmode=query&amp;dataid=$hitid&amp;type=layer");
      my $url_query_contact = selflink("view=structure&amp;viewmode=query&amp;dataid=$hitid&amp;type=contact");
      my $url_match_layer   = selflink("view=structure&amp;viewmode=match&amp;dataid=$hitid&amp;type=layer");
      my $url_match_contact = selflink("view=structure&amp;viewmode=match&amp;dataid=$hitid&amp;type=contact");
      
      $hitid =~ /(\w{4})(?:\.(\d+))?(?:\.(\w+))?/;
      my $hit_chainid = "$3";

      
      unless ($structure_loaded)
      {
        if ((!$dataid_is_complete and (!$transmembrane_chain or $hitid eq $transmembrane_chain)) or ($dataid eq $hitid))
        {
          $structure_loaded = 1;
          $url_strucA = $url_query_contact;
          $url_strucB = $url_match_contact;
          $url_align  = selflink("view=sequence&amp;dataid=$hitid");
        }
      }
      
      print qq(
      <tr class="shade$shade">
        <td>
          $pdbid
        </td>
        <td>
          $hit_chainid
        </td>
        <td>
          $hitid
        </td>
        <td>
          <a target="SequenceViewer" href=").selflink("dataid=$hitid&amp;view=sequence").qq(" onclick="parent['StructureViewerLeft'].location.href = '$url_query_layer';parent['StructureViewerRight'].location.href = '$url_match_layer'; return true;">Layer</a> &nbsp;
          <a target="SequenceViewer" href=").selflink("dataid=$hitid&amp;view=sequence").qq(" onclick="parent['StructureViewerLeft'].location.href = '$url_query_contact';parent['StructureViewerRight'].location.href = '$url_match_contact'; return true;">Contact</a>
        </td>
        <td>
          <a href=").selflink("view=download&amp;dataid=$hitid", "iMemDB_$hitid.tgz").qq(">All files</a> &nbsp;
          <a href=").selflink("view=downloadtem&amp;dataid=$hitid", "$hitid.tem").qq(">Tem file only</a>
        </td>
      </tr>);
    }
    
    print '
    </table>
    ';
    
    if ($url_align)
    {
      print qq{
        <script type="text/javascript">
          // <!--
          parent['SequenceViewer'].location.href = "$url_align";
          // -->
        </script>
      };
    }
    
    if ($url_strucA and $url_strucB)
    {
      print qq{
        <script type="text/javascript">
          // <!--
          parent['StructureViewerLeft'].location.href = '$url_strucA';
          parent['StructureViewerRight'].location.href = '$url_strucB';
          // -->
        </script>
      };
    }
    
    print qq(
    <p>&nbsp;</p>
    <p>
    You can now:
    </p>
    <ul>
    <li>
    <a href=").selflink("view=download&amp;dataid=$proteinid", $CONFIG{'APPNAME'}."_$proteinid.tgz").qq(">Download annotation for all chains</a>
    </li>
    </ul>
    
    <p class="note">Note: the files ending in &quot;.atm&quot; contain PDB co-ordinates.</p>
    
    </div>
    
    </body>
    </html>\n);
    
  }
  elsif ($view eq 'download')
  {
    my $chains = 'somestuffthatdoesnotexist';
    if ($dataid =~ /^\w{4}(.\d+(.\w(.\d+(.\d+)?)?)?)?$/)
    {
      my @chains = sort(glob("$dataid*"));
      $chains = "@chains";
      $chains =~ /^([\w\/\.]+)$/;
      $chains = "$1";
      if ($chains =~ /^\.*$/)
      {
        finish("Illegal data id: $dataid\n");
      }
    }
    else
    {
      finish("Illegal data id: $dataid\n");
    }
    open(my $stream, "tar zcf - $chains |") or finish("Could not open tar/gz stream for downloading.\n");
    print $q->header('application/x-Gzip');
    my $buffer;
    while (my $bytesread=read($stream,$buffer,1024))
    {
      print $buffer;
    }
    close($stream);
  }
  elsif ($view eq 'downloadtem')
  {
    my $chains = 'somestuffthatdoesnotexist';
    if ($dataid =~ /^\w{4}(.\d+(.\w(.\d+(.\d+)?)?)?)?$/)
    {
      my @chains = sort(glob("$dataid/query_annotation.tem"));
      if (@chains == 1)
      {
        $chains = $chains[0];
      }
      $chains =~ /^([\w\/\.]+)$/;
      $chains = "$1";
      if ($chains =~ /^\.*$/)
      {
        finish("Illegal data id: $dataid\n");
      }
    }
    else
    {
      finish("Illegal data id: $dataid\n");
    }
    open(my $stream, '<', "$chains") or finish("Could not open tar/gz stream for downloading.\n");
    print $q->header('text/plain');
    my $buffer;
    while (my $bytesread=read($stream,$buffer,1024))
    {
      print $buffer;
    }
    close($stream);
  }
  else
  {
    finish("Illegal view type: $view\n");
  }
  
  exit 0;
}



#########################################################################


sub finish
{
  my ($message) = @_;
  if (!defined($message))
  {
    $message = '(no crash message available)';
  }
  
  print $q->header('text/html');
  print qq(
  <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
              "http://www.w3.org/TR/html4/loose.dtd">
  <html>
    <head>
    <title>).$CONFIG{'APPNAME'}.qq(: Error</title>
    <link rel="stylesheet" type="text/css" href=").$CONFIG{'STYLESHEET'}.qq(">
    </head>
    <body>
    $CONFIG{'HTML_HEADER'}
    
    <p>&nbsp;</p>
    
    <h2>).html_pretty('Error').qq(</h2>
    
    <p>
    <em>$message</em>
    </p>
   
    <p>&nbsp;</p> 
    
    <p>
    The application encountered a problem and quit with the above error message.<br>
    If this occurred because you typed in a URL manually, please DO NOT<br>
    contact the author.<br>
    If you got here via the ).$CONFIG{'APPNAME'}.qq( front page and this keeps<br>
    occurring, even though you seem to be doing everything right,<br>
    please DO <a href="mailto:kelm\@stats.ox.ac.uk?subject=$CONFIG{'APPNAME'}%20web%20server%20problem">contact the author</a> and include your resultID, the above error<br>
    message and an exact description of what you did to get here.
    </p>
    
    <p>
    For now, you may try <a href="$CONFIG{'SERVERURL'}">submitting a new query</a>.
    </p>
    
    <p>&nbsp;</p>
    
    
    $CONFIG{'HTML_FOOTER'}
    </body>
  </html>
  \n);
  exit 0;
}
