#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import tempfile
import subprocess
import shlex
import random

from prosci.common import read_file, ExecutableDependencyError
from prosci.util.protein import Protein



###############################################################################



def get_scwrl_path():
  binpath = os.path.dirname(os.path.abspath(sys.argv[0]))
  for executable in ("Scwrl4", "scwrl3"):
    path = os.path.join(binpath, executable)
    if path and os.path.exists(path):
      return path
    p = subprocess.Popen("which %s 2>/dev/null" % (executable), shell=True, stdout=subprocess.PIPE)
    path, errors = p.communicate()
    path = path.strip()
    if path and os.path.exists(path):
      return path
  
  #raise ExecutableDependencyError("SCWRL is not installed (versions 4 or 3)")
  return None


def modelMissingSidechains(filein, fileout=None):
    input = Protein(filein)
    output = modelSidechains(input)
    
    for resin, resout in zip(input.to_residuelist(), output.to_residuelist()):
      if resin.rest and resin.CB is not None:
        resout.CB = resin.CB.copy()
        resout.rest = [a.copy() for a in resin.rest]
      elif resin.CB is not None and not resout.rest:
        resout.CB = resin.CB.copy()
    
    if fileout:
      f = open(fileout, "w")
      f.write(str(output))
      f.close()
    
    return output


def modelSidechains(filein, fileout=None):
    
    input = Protein(filein)
    
    fd, tmp_out = tempfile.mkstemp(suffix=".pdb", prefix="scwrl_out_")
    fd, tmp_in = tempfile.mkstemp(suffix=".pdb", prefix="scwrl_in_")
    fd, tmp_in2 = tempfile.mkstemp(suffix=".pdb", prefix="scwrl_in2_")
    
    try:
      f = open(tmp_in, "w")
      f.write(str(input))
      f.close()
      
      f = open(tmp_in2, "w")
      f.write(str(input.ligands))
      f.close()
      
      command = SCWRL_PATH
      if SCWRL_PATH.endswith("Scwrl4"):
        command += " -h"
      command += " -i '%s' -o '%s' -f '%s'" % (tmp_in, tmp_out, tmp_in2)
      
      #print command
      p = subprocess.Popen(shlex.split(command), stdout=subprocess.PIPE)
      p.communicate()
      
      output = Protein(tmp_out)
    finally:
      try:
        os.remove(tmp_out)
      except:
        pass
      try:
        os.remove(tmp_in)
      except:
        pass
      try:
        os.remove(tmp_in2)
      except:
        pass
    
    if not len(input) == len(output):
      if not output:
        raise ValueError("SCWRL failed to run")
      raise ValueError("SCWRL somehow changed the number of residues in the model...")
    
    for resin, resout in zip(input.to_residuelist(), output.to_residuelist()):
      b = 1.0
      occup = 1.0
      for a in resin:
        b = a.b
        occup = a.occup
        break
      for a in resout:
        a.b = b
        a.occup = occup
    
    output.add_ligands(input.ligands)
    
    if fileout:
      f = open(fileout, "w")
      f.write(str(output))
      f.close()
    
    return output


SCWRL_PATH = get_scwrl_path()
if SCWRL_PATH is None:
  raise ImportError("SCWRL is not installed")

