#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

import sys
import os
import subprocess
import tempfile


def edstats_files2list(mtzfname, pdbfname):
  assert os.path.exists(mtzfname)
  assert os.path.exists(pdbfname)
  tmp_fh, tmp_fname = tempfile.mkstemp(suffix=".table", prefix="edstats_output")
  try:
    command = "edstats.pl -hklin '%s' -xyzin '%s' -output '%s' -noerror"%(mtzfname, pdbfname, tmp_fname)
    print command
    p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    log, err = p.communicate()
    f = os.fdopen(tmp_fh)
    out = f.read().strip().replace("\r\n", "\n").replace("\r", "\n").splitlines()
    f.close()
  finally:
    os.remove(tmp_fname)
  
  if out:
    header = out[0].split()
    del out[0]
    assert header[:3] == ["RT", "CI", "RN"]
    numfields = len(header)
    header = header[3:]
  else:
    header = []
  
  output_data = []
  for line in out:
    line = line.strip()
    if not line:
      continue
    
    fields = line.split()
    if len(fields) != numfields:
      raise ValueError("Problem parsing EDSTATS output: expected each row to have same number of fields as header row (%d), but row had %d fields" % (len(header), len(fields)))
    
    res, chain, ires = fields[:3]
    try:
      ires = int(ires)
      inscode = ""
    except ValueError:
      inscode = ires[-1:]
      ires = int(ires[:-1])
    
    fields = fields[3:]
    
    for i, x in enumerate(fields):
      if x == "n/a":
        fields[i] = None
      else:
        try:
          fields[i] = int(x)
        except ValueError:
          fields[i] = float(x)
    
    output_data.append([(res, chain, ires, inscode), fields])
  
  
  # TODO: Do something smarter with log and err?
  #sys.stdout.write(log)
  #sys.stderr.write(err)
  
  return output_data, header, log, err


def edstats_files2dict(mtzfname, pdbfname):
  scores, header, log, err = edstats_files2list(mtzfname, pdbfname)
  mapping = {}
  for residue_label, values in scores:
    hash = {}
    for k, v in zip(header, values):
      hash[k] = v
    mapping[residue_label] = hash
  return mapping, log, err
