#!/bin/sh

mylocation=`dirname $0`
cd $mylocation

if ! [ -d "output" ]; then
  echo "You must run './run_test_case.sh' before its output can be verified!" 1>&2
  exit 1
fi


if [ -f output/error.log.txt ]; then
  a=`diff -B output/error.log.txt ../gui/cache/example/error.log.txt | grep "^[<>]" | grep -v "< Getting PDB file"`
  if ! [ "$a" ]; then
    echo "log file  : OK"
  else
    echo "The log file differs from the expected one. Differences are shown below:"
    echo $"$a"
    echo "
    If these differences are simply a few database hits, then your iMembrane
    database may be older than the author's (or newer if the author forgot to
    update this test case), which is not a serious problem."
  fi
else
  echo "
  The test case did not produce a log file. Something very basic is wrong.
  Maybe you don't have the linux command 'tee' installed." 1>&2
  exit 1
fi


if ! [ -f output/joy/template.tem ]; then
  "
  JOY output not found. Please installed JOY and make sure it is runnable.
  Ideally, put its executables in the <installdir>/bin directory" 1>&2
  exit 1
else
  echo "JOY       : OK"
fi


if [ -d output/imem ]; then
  a=`diff --brief  -B -r output/imem ../gui/cache/example/imem | grep -v ": parent"`
  if ! [ "$a" ]; then
    echo "iMembrane : OK"
  else
    echo "
    iMembrane has produced different (probably less) output than expected.
    Differences are shown below:"
    echo $"$a"
  fi
else
  echo "
  The 'imem' output directory is missing. Something must have gone wrong
  before calling iMembrane." 1>&2
  exit 1
fi


if [ -d output/medeller ]; then
  a=`diff --brief -B -r output/medeller ../gui/cache/example/medeller | egrep -v ": parent|\.loop0|target_template.loops.database.log|target_template.history.table"`
  if ! [ "$a" ]; then
    echo "MEDELLLER : OK"
  else
    echo "
    MEDELLER has produced different (probably less) output than expected.
    Differences are shown below:"
    echo $"$a"
  fi
else
  echo "
  The 'medeller' output directory is missing. Something must have gone wrong
  before calling the main MEDELLER programme." 1>&2
  exit 1
fi
